@@ -682,14 +682,14 @@ void dump_jbd_metadata (FILE *out, enum dump_block_type type, char *buf,
 	switch (type) {
 	case DUMP_BLOCK_INODE:
 		fprintf(out, "Inode\n");
-		ocfs2_swap_inode_to_cpu((struct ocfs2_dinode *)buf,
-					gbls.fs->fs_blocksize);
+		ocfs2_swap_inode_to_cpu(gbls.fs, (struct ocfs2_dinode *)buf);
 		dump_inode (out, (struct ocfs2_dinode *)buf);
 		fprintf (out, "\n");
 		break;
 	case DUMP_BLOCK_EXTENT_BLOCK:
 		fprintf(out, "Extent\n");
-		ocfs2_swap_extent_block_to_cpu((struct ocfs2_extent_block *)buf);
+		ocfs2_swap_extent_block_to_cpu(gbls.fs,
+					       (struct ocfs2_extent_block *)buf);
 		dump_extent_block (out, (struct ocfs2_extent_block *)buf);
 		fprintf (out, "\n");
 		break;
@@ -1011,3 +1011,20 @@ uint32_t dump_xattr_ibody(FILE *out, ocfs2_filesys *fs,
 		return 0;
 	}
 }
+
+void dump_frag(FILE *out, uint64_t ino, uint32_t clusters,
+	       uint32_t extents)
+{
+	float frag_level = 0;
+	int clusters_per_mb =
+		ocfs2_clusters_in_bytes(gbls.fs, OCFS2_MAX_CLUSTERSIZE);
+
+	if (clusters > 1 && extents) {
+		float e = extents, c = clusters;
+		frag_level = 100 * (e / c);
+	}
+
+	fprintf(out, "Inode: %"PRIu64"\t%% fragmented: %.2f\tclusters:"
+		" %u\textents: %u\tscore: %.0f\n", ino,
+		frag_level, clusters, extents, frag_level * clusters_per_mb);
+}
