@@ -53,6 +53,7 @@
 #include <string.h>
 #include <inttypes.h>
 #include <signal.h>
+#include <libgen.h>
 
 #include "ocfs2/ocfs2.h"
 
@@ -165,13 +166,6 @@ static errcode_t o2fsck_state_init(ocfs2_filesys *fs, o2fsck_state *ost)
 		return ret;
 	}
 
-	ret = ocfs2_block_bitmap_new(fs, "inodes with bad fields", 
-				     &ost->ost_bad_inodes);
-	if (ret) {
-		com_err(whoami, ret, "while allocating bad inodes bitmap");
-		return ret;
-	}
-
 	ret = ocfs2_block_bitmap_new(fs, "directory inodes", 
 				     &ost->ost_dir_inodes);
 	if (ret) {
@@ -186,8 +180,8 @@ static errcode_t o2fsck_state_init(ocfs2_filesys *fs, o2fsck_state *ost)
 		return ret;
 	}
 
-	ret = ocfs2_block_bitmap_new(fs, "allocated clusters",
-				     &ost->ost_allocated_clusters);
+	ret = ocfs2_cluster_bitmap_new(fs, "allocated clusters",
+				       &ost->ost_allocated_clusters);
 	if (ret) {
 		com_err(whoami, ret, "while allocating a bitmap to track "
 			"allocated clusters");
@@ -201,9 +195,6 @@ errcode_t o2fsck_state_reinit(ocfs2_filesys *fs, o2fsck_state *ost)
 {
 	errcode_t ret;
 
-	ocfs2_bitmap_free(ost->ost_bad_inodes);
-	ost->ost_bad_inodes = NULL;
-
 	ocfs2_bitmap_free(ost->ost_dir_inodes);
 	ost->ost_dir_inodes = NULL;
 
@@ -213,6 +204,11 @@ errcode_t o2fsck_state_reinit(ocfs2_filesys *fs, o2fsck_state *ost)
 	ocfs2_bitmap_free(ost->ost_allocated_clusters);
 	ost->ost_allocated_clusters = NULL;
 
+	if (ost->ost_duplicate_clusters) {
+		ocfs2_bitmap_free(ost->ost_duplicate_clusters);
+		ost->ost_duplicate_clusters = NULL;
+	}
+
 	o2fsck_icount_free(ost->ost_icount_in_inodes);
 	ost->ost_icount_in_inodes = NULL;
 
@@ -402,25 +398,9 @@ static void mark_magical_clusters(o2fsck_state *ost)
 		o2fsck_mark_clusters_allocated(ost, 0, cluster);
 }
 
-static void version(void)
+static void print_version(void)
 {
-	char url[] = "$URL$";
-       	char rev[] = "$Rev$";
-	char noise[] = "fsck.ocfs2/fsck.c";
-	char *found;
-
-	/* url =~ s/noise// :P */
-	found = strstr(url, noise);
-	if (found) {
-		char *rest = found + strlen(noise);
-		memcpy(found, rest, sizeof(url) - (found - url));
-	}
-
-	printf("fsck.ocfs2 version information from Subversion:\n"
-	       " %s\n"
-	       " %s\n", url, rev);
-
-	exit(FSCK_USAGE);
+	fprintf(stderr, "%s %s\n", whoami, VERSION);
 }
 
 static errcode_t open_and_check(o2fsck_state *ost, char *filename,
@@ -652,6 +632,8 @@ int main(int argc, char **argv)
 	int fsck_mask = FSCK_OK;
 	int slot_recover_err = 0;
 	errcode_t ret;
+	int mount_flags;
+	int proceed = 1;
 
 	memset(ost, 0, sizeof(o2fsck_state));
 	ost->ost_ask = 1;
@@ -727,7 +709,8 @@ int main(int argc, char **argv)
 				break;
 
 			case 'V':
-				version();
+				print_version();
+				exit(FSCK_USAGE);
 				break;
 
 			case 'r':
@@ -742,9 +725,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (ost->ost_skip_o2cb)
-		printf("-F given, *not* checking with the cluster DLM.\n");
-
 	if (blksize % OCFS2_MIN_BLOCKSIZE) {
 		fprintf(stderr, "Invalid blocksize: %"PRId64"\n", blksize);
 		fsck_mask |= FSCK_USAGE;
@@ -761,6 +741,45 @@ int main(int argc, char **argv)
 
 	filename = argv[optind];
 
+	print_version();
+
+	ret = ocfs2_check_if_mounted(filename, &mount_flags);
+	if (ret) {
+		com_err(whoami, ret, "while determining whether %s is mounted.",
+			filename);
+		fsck_mask |= FSCK_ERROR;
+		goto out;
+	}
+
+	if (mount_flags & (OCFS2_MF_MOUNTED | OCFS2_MF_BUSY)) {
+		if (!(open_flags & OCFS2_FLAG_RW))
+			fprintf(stdout, "\nWARNING!!! Running fsck.ocfs2 (read-"
+				"only) on a mounted filesystem may detect "
+				"invalid errors.\n\n");
+		else
+			fprintf(stdout, "\nWARNING!!! Running fsck.ocfs2 on a "
+				"mounted filesystem may cause SEVERE "
+				"filesystem damage.\n\n");
+		proceed = 0;
+	}
+
+	if (proceed && ost->ost_skip_o2cb) {
+		fprintf(stdout, "\nWARNING!!! You have disabled the cluster check. "
+			"Continue only if you\nare absolutely sure that NO "
+			"node has this filesystem mounted or is\notherwise "
+			"accessing it. If unsure, do NOT continue.\n\n");
+		proceed = 0;
+	}
+
+	if (!proceed) {
+		fprintf(stdout, "Do you really want to continue (y/N): ");
+		if (toupper(getchar()) != 'Y') {
+			printf("Aborting operation.\n");
+			fsck_mask |= FSCK_CANCELED;
+			goto out;
+		}
+	}
+
 	if (signal(SIGTERM, handle_signal) == SIG_ERR) {
 		com_err(whoami, 0, "Could not set SIGTERM");
 		exit(1);
@@ -837,6 +856,9 @@ int main(int argc, char **argv)
 	printf("  max slots:          %u\n\n", 
 	       OCFS2_RAW_SB(ost->ost_fs->fs_super)->s_max_slots);
 
+	/* Let's get enough of a cache to replay the journals */
+	o2fsck_init_cache(ost, O2FSCK_CACHE_MODE_JOURNAL);
+
 	if (open_flags & OCFS2_FLAG_RW) {
 		ret = o2fsck_check_journals(ost);
 		if (ret) {
@@ -854,6 +876,9 @@ int main(int argc, char **argv)
 		goto unlock;
 	}
 
+	/* Grow the cache */
+	o2fsck_init_cache(ost, O2FSCK_CACHE_MODE_FULL);
+
 	/* allocate all this junk after we've replayed the journal and the
 	 * sb should be stable */
 	if (o2fsck_state_init(ost->ost_fs, ost)) {
