@@ -38,6 +38,10 @@
  *  - bitmap of which inodes are directories or regular files
  *  - directory blocks that it finds off of directory inodes
  *
+ * Pass 1 also compiles a bitmap of all clusters used by the filesystem.
+ * If any clusters are shared by more than one inode, a bitmap of
+ * duplicate clusters is also created.
+ *
  * The end of Pass 1 is when the found block bitmap should contain all the
  * blocks in the system that are in use.  This is used to derive the set of
  * clusters that should be allocated.  The cluster chain allocator is loaded
@@ -63,6 +67,7 @@
 #include "icount.h"
 #include "fsck.h"
 #include "pass1.h"
+#include "pass1b.h"
 #include "problem.h"
 #include "util.h"
 #include "xattr.h"
@@ -509,11 +514,11 @@ static void o2fsck_verify_inode_fields(ocfs2_filesys *fs,
 	}
 
 	if (S_ISDIR(di->i_mode)) {
-		ocfs2_bitmap_set(ost->ost_dir_inodes, blkno, NULL);
+		o2fsck_bitmap_set(ost->ost_dir_inodes, blkno, NULL);
 		o2fsck_add_dir_parent(&ost->ost_dir_parents, blkno, 0, 0,
 				      di->i_flags & OCFS2_ORPHANED_FL);
 	} else if (S_ISREG(di->i_mode)) {
-		ocfs2_bitmap_set(ost->ost_reg_inodes, blkno, NULL);
+		o2fsck_bitmap_set(ost->ost_reg_inodes, blkno, NULL);
 	} else if (S_ISLNK(di->i_mode)) {
 		/* we only make sure a link's i_size matches
 		 * the link names length in the file data later when
@@ -887,7 +892,7 @@ size_cluster_check:
 
 		/* i_size is checked for symlinks elsewhere */
 		if (di->i_size > max_inline &&
-		    prompt(ost, PY, PR_INODE_SIZE, "Inode %"PRIu64
+		    prompt(ost, PY, PR_INODE_INLINE_SIZE, "Inode %"PRIu64
 			   "has a size of %"PRIu64" which exceeds the max "
 			   "inline data size %u. "
 			   "Correct the file size?",
@@ -898,7 +903,7 @@ size_cluster_check:
 		}
 
 		if (di->i_clusters > 0 &&
-		    prompt(ost, PY, PR_INODE_CLUSTERS,
+		    prompt(ost, PY, PR_INODE_INLINE_CLUSTERS,
 			   "Inode %"PRIu64" has %"PRIu32" clusters but it has "
 			   "inline data flag set. "
 			   "Correct the number of clusters?",
@@ -916,7 +921,7 @@ size_cluster_check:
 
 			/* i_size is checked for symlinks elsewhere */
 			if (!S_ISLNK(di->i_mode) && di->i_size <= unexpected &&
-			    prompt(ost, PY, PR_INODE_SIZE, "Inode %"PRIu64
+			    prompt(ost, PY, PR_INODE_SPARSE_SIZE, "Inode %"PRIu64
 				   " has a size of %"PRIu64" but has %"PRIu64
 				   " blocks of actual data. "
 				   "Correct the file size?",
@@ -931,7 +936,7 @@ size_cluster_check:
 			expected = ocfs2_clusters_in_blocks(fs, vb.vb_num_blocks);
 
 		if (di->i_clusters != expected &&
-		    prompt(ost, PY, PR_INODE_CLUSTERS,
+		    prompt(ost, PY, PR_INODE_SPARSE_CLUSTERS,
 			   "Inode %"PRIu64" has %"PRIu32" clusters but its "
 			   "blocks fit in %"PRIu64" clusters. "
 			   "Correct the number of clusters?",
@@ -1377,7 +1382,7 @@ errcode_t o2fsck_pass1(o2fsck_state *ost)
 		if (!memcmp(di->i_signature, OCFS2_INODE_SIGNATURE,
 			    strlen(OCFS2_INODE_SIGNATURE))) {
 
-			ocfs2_swap_inode_to_cpu(di, fs->fs_blocksize);
+			ocfs2_swap_inode_to_cpu(fs, di);
 
 			 /* We only consider inodes whose generations don't
 			  * match if the user has asked us to */
@@ -1414,6 +1419,9 @@ out_close_scan:
 out_free:
 	ocfs2_free(&buf);
 
+	if (!ret && ost->ost_duplicate_clusters)
+		ret = ocfs2_pass1_dups(ost);
+
 out:
 	return ret;
 }
