@@ -3,7 +3,7 @@
  *
  * Function prototypes, macros, etc. for related 'C' files
  *
- * Copyright (C) 2004 Oracle.  All rights reserved.
+ * Copyright (C) 2006 Oracle.  All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -20,12 +20,20 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 021110-1307, USA.
  *
- * Authors: Sunil Mushran
  */
 
 #ifndef __CORRUPT_H
 #define __CORRUPT_H
 
 void corrupt_chains(ocfs2_filesys *fs, int code, uint16_t slotnum);
+void corrupt_file(ocfs2_filesys *fs, enum fsck_type type, uint16_t slotnum);
+void corrupt_sys_file(ocfs2_filesys *fs, enum fsck_type type, uint16_t slotnum);
+void corrupt_group_desc(ocfs2_filesys *fs, enum fsck_type type,
+			uint16_t slotnum);
+void corrupt_inode(ocfs2_filesys *fs, enum fsck_type type, uint16_t slotnum);
+void corrupt_local_alloc(ocfs2_filesys *fs, enum fsck_type type,
+			 uint16_t slotnum);
+void corrupt_truncate_log(ocfs2_filesys *fs, enum fsck_type type,
+			  uint16_t slotnum);
 
 #endif		/* __CORRUPT_H */
