@@ -40,6 +40,7 @@
 #include <time.h>
 #include <string.h>
 #include <stddef.h>
+#include <stdbool.h>
 
 #include <limits.h>
 
@@ -222,13 +223,32 @@ errcode_t io_set_blksize(io_channel *channel, int blksize);
 int io_get_blksize(io_channel *channel);
 int io_get_fd(io_channel *channel);
 
-/* use ocfs2_read_blocks if your application might handle o2image file */
+/*
+ * Raw I/O functions.  They will use the I/O cache if available.  The
+ * _nocache version will not add a block to the cache, but if the block is
+ * already in the cache it will be moved to the end of the LRU and kept
+ * in a good state.
+ *
+ * Use ocfs2_read_blocks() if your application might handle o2image file.
+ *
+ * If a channel is set to 'nocache' via io_set_nocache(), it will use
+ * the _nocache() functions even if called via the regular functions.
+ * This allows control of naive code that we don't want to have to carry
+ * nocache parameters around.  Smarter code can ignore this function and
+ * use the _nocache() functions directly.
+ */
 errcode_t io_read_block(io_channel *channel, int64_t blkno, int count,
 			char *data);
+errcode_t io_read_block_nocache(io_channel *channel, int64_t blkno, int count,
+			char *data);
 errcode_t io_write_block(io_channel *channel, int64_t blkno, int count,
 			 const char *data);
+errcode_t io_write_block_nocache(io_channel *channel, int64_t blkno, int count,
+			 const char *data);
 errcode_t io_init_cache(io_channel *channel, size_t nr_blocks);
+void io_set_nocache(io_channel *channel, bool nocache);
 errcode_t io_init_cache_size(io_channel *channel, size_t bytes);
+errcode_t io_mlock_cache(io_channel *channel);
 void io_destroy_cache(io_channel *channel);
 
 errcode_t ocfs2_read_super(ocfs2_filesys *fs, uint64_t superblock, char *sb);
@@ -238,11 +258,14 @@ errcode_t ocfs2_write_primary_super(ocfs2_filesys *fs);
 errcode_t ocfs2_write_super(ocfs2_filesys *fs);
 
 /*
- * ocfs2_read_blocks is a wraper around io_read_block. If device is an image-
- * file it translates disk offset to image offset
+ * ocfs2_read_blocks() is a wraper around io_read_block. If device is an
+ * image file it translates disk offset to image offset.
+ * ocfs2_read_blocks_nocache() calls io_read_block_nocache().
  */
 errcode_t ocfs2_read_blocks(ocfs2_filesys *fs, int64_t blkno, int count,
 			    char *data);
+errcode_t ocfs2_read_blocks_nocache(ocfs2_filesys *fs, int64_t blkno, int count,
+				    char *data);
 int ocfs2_mount_local(ocfs2_filesys *fs);
 errcode_t ocfs2_open(const char *name, int flags,
 		     unsigned int superblock, unsigned int blksize,
@@ -251,8 +274,8 @@ errcode_t ocfs2_flush(ocfs2_filesys *fs);
 errcode_t ocfs2_close(ocfs2_filesys *fs);
 void ocfs2_freefs(ocfs2_filesys *fs);
 
-void ocfs2_swap_inode_from_cpu(struct ocfs2_dinode *di, size_t blocksize);
-void ocfs2_swap_inode_to_cpu(struct ocfs2_dinode *di, size_t blocksize);
+void ocfs2_swap_inode_from_cpu(ocfs2_filesys *fs, struct ocfs2_dinode *di);
+void ocfs2_swap_inode_to_cpu(ocfs2_filesys *fs, struct ocfs2_dinode *di);
 errcode_t ocfs2_read_inode(ocfs2_filesys *fs, uint64_t blkno,
 			   char *inode_buf);
 errcode_t ocfs2_write_inode(ocfs2_filesys *fs, uint64_t blkno,
@@ -266,8 +289,17 @@ errcode_t ocfs2_write_cached_inode(ocfs2_filesys *fs,
 errcode_t ocfs2_free_cached_inode(ocfs2_filesys *fs,
 				  ocfs2_cached_inode *cinode);
 
-void ocfs2_swap_extent_list_from_cpu(struct ocfs2_extent_list *el);
-void ocfs2_swap_extent_list_to_cpu(struct ocfs2_extent_list *el);
+/*
+ * obj is the object containing the extent list.  eg, if you are swapping
+ * an inode's extent list, you're passing 'di' for the obj and
+ * '&di->id2.i_list' for the el.  obj is needed to make sure the
+ * byte swapping code doesn't walk off the end of the buffer in the
+ * presence of corruption.
+ */
+void ocfs2_swap_extent_list_from_cpu(ocfs2_filesys *fs, void *obj,
+				     struct ocfs2_extent_list *el);
+void ocfs2_swap_extent_list_to_cpu(ocfs2_filesys *fs, void *obj,
+				   struct ocfs2_extent_list *el);
 errcode_t ocfs2_extent_map_get_blocks(ocfs2_cached_inode *cinode,
 				      uint64_t v_blkno, int count,
 				      uint64_t *p_blkno,
@@ -298,8 +330,10 @@ extern size_t ocfs2_journal_tag_bytes(journal_superblock_t *jsb);
 extern uint64_t ocfs2_journal_tag_block(journal_block_tag_t *tag,
 					size_t tag_bytes);
 
-void ocfs2_swap_extent_block_to_cpu(struct ocfs2_extent_block *eb);
-void ocfs2_swap_extent_block_from_cpu(struct ocfs2_extent_block *eb);
+void ocfs2_swap_extent_block_to_cpu(ocfs2_filesys *fs,
+				    struct ocfs2_extent_block *eb);
+void ocfs2_swap_extent_block_from_cpu(ocfs2_filesys *fs,
+				      struct ocfs2_extent_block *eb);
 errcode_t ocfs2_read_extent_block(ocfs2_filesys *fs, uint64_t blkno,
        				  char *eb_buf);
 errcode_t ocfs2_read_extent_block_nocheck(ocfs2_filesys *fs, uint64_t blkno,
@@ -515,6 +549,13 @@ errcode_t ocfs2_allocate_unwritten_extents(ocfs2_filesys *fs, uint64_t ino,
 					   uint64_t offset, uint64_t len);
 
 errcode_t ocfs2_truncate(ocfs2_filesys *fs, uint64_t ino, uint64_t new_i_size);
+errcode_t ocfs2_truncate_full(ocfs2_filesys *fs, uint64_t ino,
+			      uint64_t new_i_size,
+			      errcode_t (*free_clusters)(ocfs2_filesys *fs,
+							 uint32_t len,
+							 uint64_t start,
+							 void *free_data),
+			      void *free_data);
 errcode_t ocfs2_zero_tail_and_truncate(ocfs2_filesys *fs,
 				       ocfs2_cached_inode *ci,
 				       uint64_t new_size,
@@ -973,6 +1014,27 @@ static inline int ocfs2_support_xattr(struct ocfs2_super_block *osb)
 	return 0;
 }
 
+/*
+ * When we're swapping some of our disk structures, a garbage count
+ * can send us past the edge of a block buffer.  This function guards
+ * against that.  It returns true if the element would walk off then end
+ * of the block buffer.
+ */
+static inline int ocfs2_swap_barrier(ocfs2_filesys *fs, void *block_buffer,
+				     void *element, size_t element_size)
+{
+	char *limit, *end;
+
+	limit = block_buffer;
+	limit += fs->fs_blocksize;
+
+	end = element;
+	end += element_size;
+
+	return end > limit;
+}
+
+
 /*
  * shamelessly lifted from the kernel
  *
@@ -1098,10 +1160,15 @@ int ocfs2_xattr_find_leaf(ocfs2_filesys *fs, struct ocfs2_xattr_block *xb,
 			  uint32_t cpos, char **leaf_buf);
 uint16_t ocfs2_xattr_buckets_per_cluster(ocfs2_filesys *fs);
 uint16_t ocfs2_blocks_per_xattr_bucket(ocfs2_filesys *fs);
-void ocfs2_swap_xattrs_to_cpu(struct ocfs2_xattr_header *xh);
-void ocfs2_swap_xattrs_from_cpu(struct ocfs2_xattr_header *xh);
-void ocfs2_swap_xattr_block_to_cpu(struct ocfs2_xattr_block *xb);
-void ocfs2_swap_xattr_block_from_cpu(struct ocfs2_xattr_block *xb);
+/* See ocfs2_swap_extent_list() for a discussion of obj */
+void ocfs2_swap_xattrs_to_cpu(ocfs2_filesys *fs, void *obj,
+			      struct ocfs2_xattr_header *xh);
+void ocfs2_swap_xattrs_from_cpu(ocfs2_filesys *fs, void *obj,
+				struct ocfs2_xattr_header *xh);
+void ocfs2_swap_xattr_block_to_cpu(ocfs2_filesys *fs,
+				   struct ocfs2_xattr_block *xb);
+void ocfs2_swap_xattr_block_from_cpu(ocfs2_filesys *fs,
+				     struct ocfs2_xattr_block *xb);
 errcode_t ocfs2_read_xattr_block(ocfs2_filesys *fs,
 				 uint64_t blkno,
 				 char *xb_buf);
