@@ -120,12 +120,38 @@ static int ocfs2_clusters_per_group(int block_size, int cluster_size_bits)
 	return (megabytes << ONE_MB_SHIFT) >> cluster_size_bits;
 }
 
+static inline void ocfs2_zero_dinode_id2(int blocksize,
+					 struct ocfs2_dinode *di)
+{
+	memset(&di->id2, 0, blocksize - offsetof(struct ocfs2_dinode, id2));
+}
+
+void ocfs2_dinode_new_extent_list(ocfs2_filesys *fs,
+				  struct ocfs2_dinode *di)
+{
+	ocfs2_zero_dinode_id2(fs->fs_blocksize, di);
+
+	di->id2.i_list.l_tree_depth = 0;
+	di->id2.i_list.l_next_free_rec = 0;
+	di->id2.i_list.l_count = ocfs2_extent_recs_per_inode(fs->fs_blocksize);
+}
+
+void ocfs2_set_inode_data_inline(ocfs2_filesys *fs, struct ocfs2_dinode *di)
+{
+	struct ocfs2_inline_data *idata = &di->id2.i_data;
+
+	ocfs2_zero_dinode_id2(fs->fs_blocksize, di);
+
+	idata->id_count = ocfs2_max_inline_data(fs->fs_blocksize);
+
+	di->i_dyn_features |= OCFS2_INLINE_DATA_FL;
+}
+
 static void ocfs2_init_inode(ocfs2_filesys *fs, struct ocfs2_dinode *di,
 			     int16_t slot, uint64_t gd_blkno,
 			     uint64_t blkno, uint16_t mode,
 			     uint32_t flags)
 {
-	struct ocfs2_extent_list *fel;
 	int cs_bits = OCFS2_RAW_SB(fs->fs_super)->s_clustersize_bits;
 	unsigned int tl_recs;
 
@@ -171,10 +197,11 @@ static void ocfs2_init_inode(ocfs2_filesys *fs, struct ocfs2_dinode *di,
 	if (flags & OCFS2_SUPER_BLOCK_FL)
 		return ;
 
-	fel = &di->id2.i_list;
-	fel->l_tree_depth = 0;
-	fel->l_next_free_rec = 0;
-	fel->l_count = ocfs2_extent_recs_per_inode(fs->fs_blocksize);
+	if (ocfs2_support_inline_data(OCFS2_RAW_SB(fs->fs_super)) &&
+	    S_ISDIR(di->i_mode))
+		ocfs2_set_inode_data_inline(fs, di);
+	else
+		ocfs2_dinode_new_extent_list(fs, di);
 }
 
 static void ocfs2_init_eb(ocfs2_filesys *fs,
@@ -389,6 +416,39 @@ out:
 	return ret;
 }
 
+errcode_t ocfs2_delete_xattr_block(ocfs2_filesys *fs, uint64_t blkno)
+{
+	errcode_t ret;
+	char *buf;
+	int slot;
+	struct ocfs2_xattr_block *xb = NULL;
+
+	ret = ocfs2_malloc_block(fs->fs_io, &buf);
+	if (ret)
+		return ret;
+
+	ret = ocfs2_read_xattr_block(fs, blkno, buf);
+	if (ret)
+		goto out;
+
+	xb = (struct ocfs2_xattr_block *)buf;
+	slot = xb->xb_suballoc_slot;
+
+	ret = ocfs2_load_allocator(fs, EXTENT_ALLOC_SYSTEM_INODE, slot,
+				   &fs->fs_eb_allocs[slot]);
+	if (ret)
+		goto out;
+
+	ret = ocfs2_chain_free_with_io(fs, fs->fs_eb_allocs[slot], blkno);
+	if (ret)
+		goto out;
+
+out:
+	ocfs2_free(&buf);
+
+	return ret;
+}
+
 errcode_t ocfs2_delete_extent_block(ocfs2_filesys *fs, uint64_t blkno)
 {
 	errcode_t ret;
@@ -464,6 +524,45 @@ out:
 	return ret;
 }
 
+errcode_t ocfs2_test_cluster_allocated(ocfs2_filesys *fs, uint32_t cpos,
+				       int *is_allocated)
+{
+	errcode_t ret;
+	ret = ocfs2_load_allocator(fs, GLOBAL_BITMAP_SYSTEM_INODE,
+				   0, &fs->fs_cluster_alloc);
+	if (!ret) {
+		ret = ocfs2_chain_test(fs, fs->fs_cluster_alloc, cpos,
+				       is_allocated);
+	}
+
+	return ret;
+}
+
+errcode_t ocfs2_new_specific_cluster(ocfs2_filesys *fs, uint32_t cpos)
+{
+	errcode_t ret;
+	int allocatedp = 0;
+
+	/* Loads the allocator if we need it */
+	ret = ocfs2_test_cluster_allocated(fs, cpos, &allocatedp);
+	if (ret)
+		goto out;
+
+	if (allocatedp) {
+		ret = OCFS2_ET_BIT_NOT_FOUND;
+		goto out;
+	}
+
+	ocfs2_chain_force_val(fs, fs->fs_cluster_alloc, cpos, 1, NULL);
+	ret = ocfs2_write_chain_allocator(fs, fs->fs_cluster_alloc);
+	if (ret)
+		ocfs2_free_clusters(fs, 1,
+				    ocfs2_blocks_to_clusters(fs, cpos));
+
+out:
+	return ret;
+}
+
 errcode_t ocfs2_free_clusters(ocfs2_filesys *fs,
 			      uint32_t len,
 			      uint64_t start_blkno)
@@ -486,6 +585,48 @@ out:
 	return ret;
 }
 
+/*
+ * Test whether clusters have the specified value in the bitmap.
+ * test: expected value
+ * matches: 1 if all bits match test, else 0
+ */
+errcode_t ocfs2_test_clusters(ocfs2_filesys *fs,
+			      uint32_t len,
+			      uint64_t start_blkno,
+			      int test,
+			      int *matches)
+{
+	errcode_t ret;
+	uint32_t start_cluster;
+	int set = 0;
+
+	*matches = 0;
+
+	if (!len)
+		return 0;
+
+	ret = ocfs2_load_allocator(fs, GLOBAL_BITMAP_SYSTEM_INODE,
+				   0, &fs->fs_cluster_alloc);
+	if (ret)
+		goto out;
+
+	start_cluster = ocfs2_blocks_to_clusters(fs, start_blkno);
+
+	while (len) {
+		ret = ocfs2_bitmap_test(fs->fs_cluster_alloc->ci_chains,
+					start_cluster, &set);
+		if (ret || set != test)
+			goto out;
+
+		len--;
+		start_cluster++;
+	}
+
+	*matches = 1;
+out:
+	return ret;
+}
+
 #ifdef DEBUG_EXE
 #include <stdio.h>
 
