@@ -27,7 +27,7 @@
 
 #include <string.h>
 
-#include "ocfs2.h"
+#include "ocfs2/ocfs2.h"
 
 errcode_t ocfs2_read_cached_inode(ocfs2_filesys *fs, uint64_t blkno,
 				  ocfs2_cached_inode **ret_ci)
@@ -73,9 +73,6 @@ errcode_t ocfs2_free_cached_inode(ocfs2_filesys *fs,
 	if (!cinode)
 		return OCFS2_ET_INVALID_ARGUMENT;
 	
-	if (cinode->ci_map)
-		ocfs2_extent_map_free(cinode);
-	
 	if (cinode->ci_chains)
 		ocfs2_bitmap_free(cinode->ci_chains);
 
