@@ -29,11 +29,10 @@
 #include <string.h>
 #include <inttypes.h>
 
-#include "ocfs2.h"
+#include "ocfs2/ocfs2.h"
 
 #include "bitmap.h"
-#include "bitops.h"
-#include "kernel-rbtree.h"
+#include "ocfs2/bitops.h"
 
 
 struct chainalloc_bitmap_private {
@@ -486,7 +485,7 @@ void ocfs2_init_group_desc(ocfs2_filesys *fs,
 {
 	memset(gd, 0, fs->fs_blocksize);
 
-	strcpy(gd->bg_signature, OCFS2_GROUP_DESC_SIGNATURE);
+	strcpy((char *)gd->bg_signature, OCFS2_GROUP_DESC_SIGNATURE);
 	gd->bg_generation = generation;
 	gd->bg_size = ocfs2_group_bitmap_size(fs->fs_blocksize);
 	gd->bg_bits = bits;
