@@ -41,6 +41,7 @@
 #include <sys/resource.h>
 #include <sys/utsname.h>
 #endif
+#include <sys/mman.h>
 #include <inttypes.h>
 
 #include "ocfs2/kernel-rbtree.h"
@@ -48,6 +49,14 @@
 #include "ocfs2/ocfs2.h"
 
 
+/*
+ * We do cached I/O in 1MB hunks, so we need this constant.
+ */
+#ifndef ONE_MEGABYTE
+# define ONE_MEGABYTE (1024 * 1024)
+#endif
+
+
 /*
  * The cache looks up blocks in two ways:
  *
@@ -71,7 +80,10 @@ struct io_cache {
 
 	/* Housekeeping */
 	struct io_cache_block *ic_metadata_buffer;
+	unsigned long ic_metadata_buffer_len;
 	char *ic_data_buffer;
+	unsigned long ic_data_buffer_len;
+	int ic_locked;
 };
 
 struct _io_channel {
@@ -80,9 +92,20 @@ struct _io_channel {
 	int io_flags;
 	int io_error;
 	int io_fd;
+	bool io_nocache;
 	struct io_cache *io_cache;
 };
 
+/*
+ * We open code this because we don't have the ocfs2_filesys to call
+ * ocfs2_blocks_in_bytes().
+ */
+static inline int one_meg_of_blocks(io_channel *channel)
+{
+	int count = ONE_MEGABYTE + channel->io_blksize - 1;
+	return count / channel->io_blksize;
+}
+
 static errcode_t unix_io_read_block(io_channel *channel, int64_t blkno,
 				    int count, char *data)
 {
@@ -121,8 +144,9 @@ out:
 	return ret;
 }
 
-static errcode_t unix_io_write_block(io_channel *channel, int64_t blkno,
-				     int count, const char *data)
+static errcode_t unix_io_write_block_full(io_channel *channel, int64_t blkno,
+					  int count, const char *data,
+					  int *completed)
 {
 	int ret;
 	ssize_t size, tot, wr;
@@ -150,13 +174,19 @@ static errcode_t unix_io_write_block(io_channel *channel, int64_t blkno,
 
 	ret = 0;
 out:
+	if (completed)
+		*completed = tot / channel->io_blksize;
 	if (!ret && (tot != size))
 		ret = OCFS2_ET_SHORT_WRITE;
 
 	return ret;
 }
 
-
+static errcode_t unix_io_write_block(io_channel *channel, int64_t blkno,
+				     int count, const char *data)
+{
+	return unix_io_write_block_full(channel, blkno, count, data, NULL);
+}
 
 /*
  * See if the rbtree has a block for the given block number.
@@ -214,6 +244,17 @@ static void io_cache_seen(struct io_cache *ic, struct io_cache_block *icb)
 	list_add_tail(&icb->icb_list, &ic->ic_lru);
 }
 
+static void io_cache_unsee(struct io_cache *ic, struct io_cache_block *icb)
+{
+	/*
+	 * Move to the end of the LRU.  There's no point in removing an
+	 * "unseen" buffer from the cache.  It's valid, but we want the
+	 * next I/O to steal it.
+	 */
+	list_del(&icb->icb_list);
+	list_add(&icb->icb_list, &ic->ic_lru);
+}
+
 static void io_cache_disconnect(struct io_cache *ic,
 				struct io_cache_block *icb)
 {
@@ -238,106 +279,199 @@ static struct io_cache_block *io_cache_pop_lru(struct io_cache *ic)
 	return icb;
 }
 
-static errcode_t io_cache_read_one_block(io_channel *channel, int64_t blkno,
-					 char *data)
+/*
+ * This relies on the fact that our cache is always up to date.  If a
+ * block is in the cache, the same thing is on disk.  Even if we re-read
+ * the disk block, we don't need to update the cache.  This allows us
+ * to look for optimal I/O sizes; it's better to call one read 1MB of
+ * half-cached blocks than to read every other block.
+ *
+ * If the caller specifies "nocache", we still want to give them anything
+ * we found in the cache, but we want cached blocks moved to the front
+ * of the LRU.  That way they get stolen first.
+ */
+static errcode_t io_cache_read_blocks(io_channel *channel, int64_t blkno,
+				      int count, char *data, bool nocache)
 {
+	int i, good_blocks;
 	errcode_t ret = 0;
 	struct io_cache *ic = channel->io_cache;
 	struct io_cache_block *icb;
 
-	icb = io_cache_lookup(ic, blkno);
-	if (icb)
-		goto found;
-
-	/* Ok, this blkno isn't in the cache.  Steal something. */
-	icb = io_cache_pop_lru(ic);
-
 	/*
-	 * If the read fails, we leave the block at the end of the LRU
-	 * and out of the lookup tree.
+	 * Here we check two things:
+	 *
+	 * 1) Are all the blocks cached?  If so, we can skip I/O.
+	 * 2) If they are not all cached, we want to start our read at the
+	 *    first uncached blkno.
 	 */
-	ret = unix_io_read_block(channel, blkno, 1, icb->icb_buf);
-	if (ret)
-		goto out;
+	for (good_blocks = 0; good_blocks < count; good_blocks++) {
+		icb = io_cache_lookup(ic, blkno + good_blocks);
+		if (!icb)
+			break;
+	}
 
-	icb->icb_blkno = blkno;
-	io_cache_insert(ic, icb);
+	/* Read any blocks not in the cache */
+	if (good_blocks < count) {
+		ret = unix_io_read_block(channel, blkno + good_blocks,
+					 count - good_blocks,
+					 data + (channel->io_blksize *
+						 good_blocks));
+		if (ret)
+			goto out;
+	}
 
-found:
-	memcpy(data, icb->icb_buf, channel->io_blksize);
-	io_cache_seen(ic, icb);
+	/* Now we sync up the cache with the data buffer */
+	for (i = 0; i < count; i++, data += channel->io_blksize) {
+		icb = io_cache_lookup(ic, blkno + i);
+		if (i < good_blocks) {
+			/*
+			 * We skipped reading this because it was in the
+			 * cache.  Copy it to the data buffer.
+			 */
+			assert(icb);
+			memcpy(data, icb->icb_buf, channel->io_blksize);
+		} else if (!icb) {
+			if (nocache)
+				continue;
+
+			/* Steal the LRU buffer */
+			icb = io_cache_pop_lru(ic);
+			icb->icb_blkno = blkno + i;
+			io_cache_insert(ic, icb);
+
+			/*
+			 * We did I/O into the data buffer, now update
+			 * the cache.
+			 */
+			memcpy(icb->icb_buf, data, channel->io_blksize);
+		}
+		/*
+		 * What about if ((i >= good_blocks) && icb)?  That means
+		 * we had the buffer in the cache, but we read it anyway
+		 * to get a single I/O.  Our cache guarantees that the
+		 * contents will match, so we just skip to marking the
+		 * buffer seen.
+		 */
+
+		if (nocache)
+			io_cache_unsee(ic, icb);
+		else
+			io_cache_seen(ic, icb);
+	}
 
 out:
 	return ret;
 }
 
 static errcode_t io_cache_read_block(io_channel *channel, int64_t blkno,
-				     int count, char *data)
+				     int count, char *data, bool nocache)
 
 {
-	int i;
+	int todo = one_meg_of_blocks(channel);
 	errcode_t ret = 0;
 
-	for (i = 0; i < count; i++, blkno++, data += channel->io_blksize) {
-		ret = io_cache_read_one_block(channel, blkno, data);
+	/*
+	 * We do this in one meg hunks so that each hunk has an
+	 * opportunity to be in cache, but we get a good throughput.
+	 */
+	while (count) {
+		if (todo > count)
+			todo = count;
+		ret = io_cache_read_blocks(channel, blkno, todo, data,
+					   nocache);
 		if (ret)
 			break;
+
+		blkno += todo;
+		count -= todo;
+		data += (channel->io_blksize * todo);
 	}
 
 	return ret;
 }
 
-static errcode_t io_cache_write_one_block(io_channel *channel,
-					  int64_t blkno, const char *data)
+/*
+ * This relies on the fact that our cache is always up to date.  If a
+ * block is in the cache, the same thing is on disk.  So here we'll write
+ * a whole stream and update the cache as needed.
+ */
+static errcode_t io_cache_write_blocks(io_channel *channel, int64_t blkno,
+				       int count, const char *data,
+				       bool nocache)
 {
+	int i, completed = 0;
 	errcode_t ret;
 	struct io_cache *ic = channel->io_cache;
 	struct io_cache_block *icb;
 
-	icb = io_cache_lookup(ic, blkno);
-	if (icb)
-		goto found;
+	/* Get the write out of the way */
+	ret = unix_io_write_block_full(channel, blkno, count, data,
+				       &completed);
 
-	/* Ok, this blkno isn't in the cache.  Steal something. */
-	icb = io_cache_pop_lru(ic);
+	/*
+	 * Now we sync up the cache with the data buffer.  We have
+	 * to sync up I/O that completed, even if the entire I/O did not.
+	 *
+	 * In the nocache case, we want to skip blocks that weren't in the
+	 * cache, but we want to update blocks that where.  Even though
+	 * the caller specified "don't cache this", it's already in the
+	 * cache.  We don't want stale data.
+	 */
+	for (i = 0; i < completed; i++, data += channel->io_blksize) {
+		icb = io_cache_lookup(ic, blkno + i);
+		if (!icb) {
+			if (nocache)
+				continue;
 
-	icb->icb_blkno = blkno;
-	io_cache_insert(ic, icb);
+			/*
+			 * Steal the LRU buffer.  We can't error here, so
+			 * we can safely insert it before we copy the data.
+			 */
+			icb = io_cache_pop_lru(ic);
+			icb->icb_blkno = blkno + i;
+			io_cache_insert(ic, icb);
+		}
 
-found:
-	memcpy(icb->icb_buf, data, channel->io_blksize);
-	io_cache_seen(ic, icb);
-
-	ret = unix_io_write_block(channel, blkno, 1, icb->icb_buf);
-	if (ret)
-		io_cache_disconnect(ic, icb);
+		memcpy(icb->icb_buf, data, channel->io_blksize);
+		if (nocache)
+			io_cache_unsee(ic, icb);
+		else
+			io_cache_seen(ic, icb);
+	}
 
 	return ret;
 }
 
 static errcode_t io_cache_write_block(io_channel *channel, int64_t blkno,
-				      int count, const char *data)
-
+				      int count, const char *data,
+				      bool nocache)
 {
-	int i;
-	errcode_t ret = 0;
-
-	for (i = 0; i < count; i++, blkno++, data += channel->io_blksize) {
-		ret = io_cache_write_one_block(channel, blkno, data);
-		if (ret)
-			break;
-	}
-
-	return ret;
+	/*
+	 * Unlike io_read_cache_block(), we're going to do all of the
+	 * I/O no matter what.  We keep the separation of
+	 * io_cache_write_block() and io_cache_write_blocks() for
+	 * consistency.
+	 */
+	return io_cache_write_blocks(channel, blkno, count, data,
+				     nocache);
 }
 
 static void io_free_cache(struct io_cache *ic)
 {
 	if (ic) {
-		if (ic->ic_data_buffer)
+		if (ic->ic_data_buffer) {
+			if (ic->ic_locked)
+				munlock(ic->ic_data_buffer,
+					ic->ic_data_buffer_len);
 			ocfs2_free(&ic->ic_data_buffer);
-		if (ic->ic_metadata_buffer)
+		}
+		if (ic->ic_metadata_buffer) {
+			if (ic->ic_locked)
+				munlock(ic->ic_metadata_buffer,
+					ic->ic_metadata_buffer_len);
 			ocfs2_free(&ic->ic_metadata_buffer);
+		}
 		ocfs2_free(&ic);
 	}
 }
@@ -350,6 +484,47 @@ void io_destroy_cache(io_channel *channel)
 	}
 }
 
+/*
+ * A cache is kind of pointless if it is swappable, right?  Let's give
+ * applications the ability to pin the cache memory.  This is a separate
+ * call from io_init_cache() because non-privileged users can't do it, and
+ * they still want to create small caches.
+ */
+errcode_t io_mlock_cache(io_channel *channel)
+{
+	int rc;
+	struct io_cache *ic = channel->io_cache;
+	long pages_wanted, avpages;
+
+	if (!ic)
+		return OCFS2_ET_INVALID_ARGUMENT;
+
+	if (ic->ic_locked)
+		return 0;
+
+	/*
+	 * We're going to lock our cache pages.  We don't want to
+	 * request more memory than the system has, though.
+	 */
+	pages_wanted = channel->io_blksize * ic->ic_nr_blocks / getpagesize();
+	avpages = sysconf(_SC_AVPHYS_PAGES);
+	if (pages_wanted > avpages)
+		return OCFS2_ET_NO_MEMORY;
+
+	rc = mlock(ic->ic_data_buffer, ic->ic_data_buffer_len);
+	if (!rc) {
+		rc = mlock(ic->ic_metadata_buffer, ic->ic_metadata_buffer_len);
+		if (rc)
+			munlock(ic->ic_data_buffer, ic->ic_data_buffer_len);
+	}
+
+	if (rc)
+		return OCFS2_ET_NO_MEMORY;
+
+	ic->ic_locked = 1;
+	return 0;
+}
+
 errcode_t io_init_cache(io_channel *channel, size_t nr_blocks)
 {
 	int i;
@@ -369,11 +544,14 @@ errcode_t io_init_cache(io_channel *channel, size_t nr_blocks)
 	ret = ocfs2_malloc_blocks(channel, nr_blocks, &ic->ic_data_buffer);
 	if (ret)
 		goto out;
+	ic->ic_data_buffer_len = (unsigned long)nr_blocks * channel->io_blksize;
 
 	ret = ocfs2_malloc0(sizeof(struct io_cache_block) * nr_blocks,
 			    &ic->ic_metadata_buffer);
 	if (ret)
 		goto out;
+	ic->ic_metadata_buffer_len =
+		(unsigned long)nr_blocks * sizeof(struct io_cache_block);
 
 	icb_list = ic->ic_metadata_buffer;
 	dbuf = ic->ic_data_buffer;
@@ -448,6 +626,7 @@ errcode_t io_open(const char *name, int flags, io_channel **channel)
 	strcpy(chan->io_name, name);
 	chan->io_blksize = OCFS2_MIN_BLOCKSIZE;
 	chan->io_flags = (flags & OCFS2_FLAG_RW) ? O_RDWR : O_RDONLY;
+	chan->io_nocache = false;
 	if (!(flags & OCFS2_FLAG_BUFFERED))
 		chan->io_flags |= O_DIRECT;
 	chan->io_error = 0;
@@ -565,11 +744,34 @@ int io_get_fd(io_channel *channel)
 	return channel->io_fd;
 }
 
+/*
+ * If a channel is set to 'nocache', it will use the _nocache() functions
+ * even if called via the regular functions.  This allows control of
+ * naive code that we don't want to have to carry nocache parameters
+ * around.  Smarter code can ignore this function and use the _nocache()
+ * functions directly.
+ */
+void io_set_nocache(io_channel *channel, bool nocache)
+{
+	channel->io_nocache = nocache;
+}
+
 errcode_t io_read_block(io_channel *channel, int64_t blkno, int count,
 			char *data)
 {
 	if (channel->io_cache)
-		return io_cache_read_block(channel, blkno, count, data);
+		return io_cache_read_block(channel, blkno, count, data,
+					   channel->io_nocache);
+	else
+		return unix_io_read_block(channel, blkno, count, data);
+}
+
+errcode_t io_read_block_nocache(io_channel *channel, int64_t blkno, int count,
+				char *data)
+{
+	if (channel->io_cache)
+		return io_cache_read_block(channel, blkno, count, data,
+					   true);
 	else
 		return unix_io_read_block(channel, blkno, count, data);
 }
@@ -578,7 +780,18 @@ errcode_t io_write_block(io_channel *channel, int64_t blkno, int count,
 			 const char *data)
 {
 	if (channel->io_cache)
-		return io_cache_write_block(channel, blkno, count, data);
+		return io_cache_write_block(channel, blkno, count, data,
+					    channel->io_nocache);
+	else
+		return unix_io_write_block(channel, blkno, count, data);
+}
+
+errcode_t io_write_block_nocache(io_channel *channel, int64_t blkno, int count,
+				 const char *data)
+{
+	if (channel->io_cache)
+		return io_cache_write_block(channel, blkno, count, data,
+					    true);
 	else
 		return unix_io_write_block(channel, blkno, count, data);
 }
