@@ -239,11 +239,7 @@ static void finish_normal_format(State *s)
 	int num;
 	ocfs2_filesys *fs;
 
-	/* These routines use libocfs2 to do their work. We
-	 * don't share an ocfs2_filesys context between the
-	 * journal format and the lost+found create so that
-	 * the library can use the journal for the latter in
-	 * future revisions. */
+	/* These routines use libocfs2 to do their work. */
 
 	ret = ocfs2_open(s->device_name, OCFS2_FLAG_RW, 0, 0, &fs);
 	if (ret) {
@@ -254,53 +250,36 @@ static void finish_normal_format(State *s)
 		exit(1);
 	}
 
-	if (!s->no_backup_super) {
-		ret = io_init_cache(fs->fs_io,
-				    ocfs2_extent_recs_per_eb(fs->fs_blocksize));
-		if (ret)
-			com_err(s->progname, ret,
-				"while initializing the I/O cache.  Continuing "
-				"without a cache (safe, but slower)");
-
-		if (!s->quiet)
-			printf("Writing backup superblock: ");
-
-		num = format_backup_super(s, fs);
-		if (!s->quiet)
-			printf("%d block(s)\n", num);
-
-		io_destroy_cache(fs->fs_io);
-	}
-
-	/* io cache is disabled during journal format for performance reasons */
-	if (!s->quiet)
-		printf("Formatting Journals: ");
-
-	format_journals(s, fs);
-
-	if (!s->quiet)
-		printf("done\n");
-
-	ret = io_init_cache(fs->fs_io,
-			    ocfs2_extent_recs_per_eb(fs->fs_blocksize));
+	/* 8MB should cover an allocator and some other stuff */
+	ret = io_init_cache_size(fs->fs_io, 8 * 1024 * 1024);
 	if (ret)
 		com_err(s->progname, ret,
 			"while initializing the I/O cache.  Continuing "
 			"without a cache (safe, but slower)");
 
+	if (!s->no_backup_super) {
+		if (!s->quiet)
+			printf("Writing backup superblock: ");
+		num = format_backup_super(s, fs);
+		if (!s->quiet)
+			printf("%d block(s)\n", num);
+	}
+
+	if (!s->quiet)
+		printf("Formatting Journals: ");
+	format_journals(s, fs);
+	if (!s->quiet)
+		printf("done\n");
+
 	if (!s->quiet)
 		printf("Formatting slot map: ");
-
 	format_slotmap(s, fs);
-
 	if (!s->quiet)
 		printf("done\n");
 
 	if (!s->quiet)
 		printf("Writing lost+found: ");
-
 	create_lost_found_dir(s, fs);
-
 	if (!s->quiet)
 		printf("done\n");
 
@@ -1226,10 +1205,14 @@ fill_defaults(State *s)
 
 	err = ocfs2_get_device_sectsize(s->device_name, &sectsize);
 	if (err) {
-		com_err(s->progname, err,
-			"while getting hardware sector size of device %s",
-			s->device_name);
-		exit(1);
+		if (err == OCFS2_ET_CANNOT_DETERMINE_SECTOR_SIZE)
+			sectsize = 0;
+		else {
+			com_err(s->progname, err,
+				"while getting hardware sector size of "
+				"device %s", s->device_name);
+			exit(1);
+		}
 	}
 	if (!sectsize)
 		sectsize = OCFS2_MIN_BLOCKSIZE;
@@ -1532,6 +1515,7 @@ initialize_bitmap(State *s, uint32_t bits, uint32_t unit_bits,
 
 	c_to_b_bits = s->cluster_size_bits - s->blocksize_bits;
 
+	/* to the next aligned cluster */
 	s->first_cluster_group = (OCFS2_SUPER_BLOCK_BLKNO + 1);
 	s->first_cluster_group += ((1 << c_to_b_bits) - 1);
 	s->first_cluster_group >>= c_to_b_bits;
@@ -1577,13 +1561,14 @@ initialize_bitmap(State *s, uint32_t bits, uint32_t unit_bits,
 		blkno += (uint64_t) s->global_cpg << (s->cluster_size_bits - s->blocksize_bits);
 		chain++;
 		if (chain >= recs_per_inode) {
-			bitmap->num_chains = recs_per_inode;
 			chain = 0;
 			wrapped = 1;
 		}
 	}
 	if (!wrapped)
 		bitmap->num_chains = chain;
+	else
+		bitmap->num_chains = recs_per_inode;
 
 	/* by now, this should be accurate. */
 	if (bm_record->bi.total_bits != s->volume_size_in_clusters) {
@@ -1945,6 +1930,15 @@ check_32bit_blocks(State *s)
 	exit(1);
 }
 
+static void mkfs_swap_inode_from_cpu(State *s, struct ocfs2_dinode *di)
+{
+	ocfs2_filesys fake_fs;
+	char super_buf[OCFS2_MAX_BLOCKSIZE];
+
+	fill_fake_fs(s, &fake_fs, super_buf);
+	ocfs2_swap_inode_from_cpu(&fake_fs, di);
+}
+
 static void
 format_superblock(State *s, SystemFileDiskRecord *rec,
 		  SystemFileDiskRecord *root_rec, SystemFileDiskRecord *sys_rec)
@@ -2025,7 +2019,7 @@ format_superblock(State *s, SystemFileDiskRecord *rec,
 	strcpy((char *)di->id2.i_super.s_label, s->vol_label);
 	memcpy(di->id2.i_super.s_uuid, s->uuid, 16);
 
-	ocfs2_swap_inode_from_cpu(di, s->blocksize);
+	mkfs_swap_inode_from_cpu(s, di);
 	mkfs_compute_meta_ecc(s, di, &di->i_check);
 	do_pwrite(s, di, s->blocksize, super_off);
 	free(di);
@@ -2191,7 +2185,7 @@ format_file(State *s, SystemFileDiskRecord *rec)
 	}
 
 write_out:
-	ocfs2_swap_inode_from_cpu(di, s->blocksize);
+	mkfs_swap_inode_from_cpu(s, di);
 	mkfs_compute_meta_ecc(s, di, &di->i_check);
 	do_pwrite(s, di, s->blocksize, rec->fe_off);
 	free(di);
