@@ -30,7 +30,7 @@
 
 #include <uuid/uuid.h>
 
-#include "ocfs2.h"
+#include "ocfs2/ocfs2.h"
 
 
 #define Filesystem_Check(op)  PyObject_TypeCheck(op, &Filesystem_Type)
@@ -176,7 +176,7 @@ dinode_repr (DInode *self)
 {
   char blkno[32];
 
-  snprintf (blkno, sizeof (blkno), "%"PRIu64, self->dinode.i_blkno);
+  snprintf (blkno, sizeof (blkno), "%"PRIu64, (uint64_t)self->dinode.i_blkno);
   return PyString_FromFormat ("<ocfs2.DInode %s on %s>", blkno,
 			      PyString_AS_STRING (self->fs_obj->device));
 }
@@ -348,7 +348,7 @@ SUPER_U64_GETTER (first_cluster_group)
 static PyObject *
 super_uuid (SuperBlock *self, void *closure)
 {
-  return PyString_FromStringAndSize (self->super.s_uuid,
+  return PyString_FromStringAndSize ((char *)self->super.s_uuid,
                                      sizeof (self->super.s_uuid));
 }
 
